;
; 22-09-2007
;
; The programm implments the summing tree algorithm.
; There are 5 variations: root, middle L, middle R, leaf L, leaf R.
; The data are input from the leafs PORTA 0,1
; The data are output from the root PORTA 0,1,3,4
;
;******************************************************************************

               .equ __24FJ16GA002, 1
               .include "p24FJ16GA002.inc"

;..............................................................................
;Program Specific Constants (literals used in code)
;..............................................................................

        .equ SAMPLES, 64         ;Number of samples


;..............................................................................
;Global Declarations:
;..............................................................................

        .global _wreg_init       ;Provide global scope to _wreg_init routine
                                 ;In order to call this routine from a C file,
                                 ;place "wreg_init" in an "extern" declaration
                                 ;in the C file.

        .global __reset          ;The label for the first line of code. 

        .global __T1Interrupt    ;Declare Timer 1 ISR name global



;..............................................................................
;Constants stored in Program space
;..............................................................................

        .section .myconstbuffer, code
        .palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2
ps_coeff:
        .hword   0x0002, 0x0003, 0x0005, 0x000A




;..............................................................................
;Uninitialized variables in Near data memory (Lower 8Kb of RAM)
;..............................................................................

          .section .nbss, bss, near
var1:     .space 2               ;Example of allocating 1 word of space for
                                 ;variable "var1".



;..............................................................................
;Code Section in Program Memory
;..............................................................................

.text                             ;Start of Code section
__reset:
        MOV #__SP_init, W15       ;Initalize the Stack Pointer
        MOV #0x17F8, W0     ;Initialize the Stack Pointer Limit Register
        MOV W0, SPLIM
        NOP                       ;Add NOP to follow SPLIM initialization
        
        CALL _wreg_init           ;Call _wreg_init subroutine
                                  ;Optionally use RCALL instead of CALL




        ;<<insert more user code here>>


;
; initialize
;
		MOV #0xFFFF,W1
		MOV W1,AD1PCFG			; turn off the Analog nature of Port A pins

; root
;		MOV	#0xFFE4, W1			; set PORTA 0,1,3,4 as output, all others as input.
;		MOV	W1,TRISA
; Middle-leaf
		MOV	#0xFFFF, W1			; set PORTA as input.
		MOV	W1,TRISA


		MOV	#0x1F0F,W1
		MOV	W1,RPINR18			; RxD --> RP15

		MOV	#0x0300,W1
		MOV	W1,RPOR5			; TxD --> RP11



		MOV	#0x0000,W1
		MOV	W1,U1BRG			; BRG <-- 0000 (max baud rate)
		MOV	#0x0008,W1
		MOV	W1,U1MODE			; setup UART
		MOV	#0x8008,W1
		MOV	W1,U1MODE			; Enable UART
		MOV	#0x04C0,W1
		MOV	W1,U1STA			; setup UART


		MOV	#0x0003,W4			;mask bits
		MOV	#0x0018,W5

;
;  W0: Temp register (configuration)
;
;  W1: Left input data
;  W2: Right input data
;  W3: Output data
;
;  W4,W5: Mask values
;
;  W6,W7: Temp registers
;  W8: Loop counter

;-----------------------------------------
;Middle R

loop:
;=================
; read-write value
NOP
NOP
NOP
NOP
NOP
;=================
; RxD --> RP15
MOV	#0x0008,W6
MOV	W6,U1MODE

MOV	#0x1F0F,W6
MOV	W6,RPINR18

MOV	#0x8008,W6
MOV	W6,U1MODE
MOV	#0x04C0,W6
MOV	W6,U1STA
;=================
NOP
;=================
MOV	#100,W8
loopW1:
NOP
DEC	W8,W8
BRA	NZ,loopW1
;=================
; read left data (W1)
MOV	U1RXREG,W1
;=================
; RxD --> RP13
MOV	#0x0008,W6
MOV	W6,U1MODE

MOV	#0x1F0D,W6
MOV	W6,RPINR18

MOV	#0x8008,W6
MOV	W6,U1MODE
MOV	#0x04C0,W6
MOV	W6,U1STA
;=================
MOV	W3,U1TXREG
;=================
MOV	#100,W8
loopW2:
NOP
DEC	W8,W8
BRA	NZ,loopW2
;=================
; read right data (W2)
MOV	U1RXREG,W2
;=================
ADD	W1,W2,W3
;=================
BRA loop




;----------------------------------

done:
        BRA     done              ;Place holder for last line of executed code



;..............................................................................
;Subroutine: Initialization of W registers to 0x0000
;..............................................................................

_wreg_init:
        CLR W0
        MOV W0, W14
        REPEAT #12
        MOV W0, [++W14]
        CLR W14
        RETURN



;..............................................................................
;Timer 1 Interrupt Service Routine
;Example context save/restore in the ISR performed using PUSH.D/POP.D
;instruction. The instruction pushes two words W4 and W5 on to the stack on
;entry into ISR and pops the two words back into W4 and W5 on exit from the ISR
;..............................................................................

__T1Interrupt:
        PUSH.D W4                  ;Save context using double-word PUSH

        ;<<insert more user code here>>

        BCLR IFS0, #T1IF           ;Clear the Timer1 Interrupt flag Status
                                   ;bit.
        POP.D W4                   ;Retrieve context POP-ping from Stack
        RETFIE                     ;Return from Interrupt Service routine



;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file



